--[[
	CORE
]]
local function debug(...)
	if false then
		print(...)
	end
end

local Poisoner = CreateFrame("FRAME", "Poisoner")

local textureprefix = "Interface\\Icons\\"
local texturesuffix = ""
local playerGUID
local currentbuffs = {}
local k, v, cat, spells, spellID,_

if true then
	textureprefix = "Interface\\AddOns\\Poisoner\\images\\"
end

local spells = {
	[2823] = "Deadly Poison",
	[8679] = "Wound Poison",
	[3408] = "Crippling Poison",
	[5761] = "Mind-numbing Poison",
	[108211] = "Leeching Poison",
	[108215] = "Paralytic Poison",
}

local poisons = {
	["Lethal"] = {
		[2823] = "Deadly Poison",
		[8679] = "Wound Poison",
	},
	["Non-Lethal"] = {
		[3408] = "Crippling Poison",
		[5761] = "Mind-numbing Poison",
		[108211] = "Leeching Poison",
		[108215] = "Paralytic Poison",
	},
}

local events = {
	["COMBAT_LOG_EVENT_UNFILTERED"] = true,
	["PLAYER_LOGIN"] = true,
	["PLAYER_ENTERING_WORLD"] = true,
	["SPELLS_CHANGED"] = true,
	["PLAYER_REGEN_DISABLED"] = true,
	["PLAYER_REGEN_ENABLED"] = true,
	["UNIT_AURA"] = true,
}

local CLEU_events = {
	["SPELL_AURA_APPLIED"] = true,
	["SPELL_AURA_REFRESH"] = true,
	["SPELL_AURA_REMOVED"] = true,
	["SPELL_CAST_SUCCESS"] = true,
	["SPELL_INTERRUPT"] = true,
	["SPELL_MISSED"] = true,
	["SPELL_UPDATE_COOLDOWN"] = true,
}

local function GetCircularPoint(angle, radius)
	
	local angle = tonumber(angle)
	local radius = tonumber(radius)
	
	while angle > 360 do
		angle = angle - 360
	end
	
	local x, y
	
	if angle == 270 then
		x = -radius
		y = 0
	elseif angle == 180 then
		x = 0
		y = -radius
	elseif angle == 90 then
		x = radius
		y = 0
	elseif angle == 0 or angle == 360 then
		x = 0
		y = radius
	elseif angle > 270 then
		x = -(radius * cos(angle-270))
		y = radius * sin(angle-270)
	elseif angle > 180 then
		x = -(radius * sin(angle-180))
		y = -(radius * cos(angle-180))
	elseif angle > 90 then
		x = radius * cos(angle-90)
		y = -(radius * sin(angle-90))
	else
		x = radius * sin(angle)
		y = radius * cos(angle)
	end
	
	return x,y
	
end

Poisoner.OnEvent = function(self, event, ...)
	if Poisoner[event] then
		Poisoner[event](self, ...)
		--print("OnEvent", event, ...)
	else
		--print("Poisoner OnEvent:", self, event, ...)
	end
end

for k,v in pairs(events) do
	Poisoner:RegisterEvent(k)
end

Poisoner:SetScript("OnEvent", Poisoner.OnEvent)


function Poisoner:PLAYER_LOGIN()
	playerGUID = UnitGUID("player")
	Poisoner:CreateFrames()
end

function Poisoner:PLAYER_ENTERING_WORLD()
	Poisoner:CheckBuffs()
end

function Poisoner:UNIT_AURA(UnitId)
	if UnitId ~= "player" then return end
	Poisoner:CheckBuffs()
	Poisoner:UnregisterEvent("UNIT_AURA")	--Only call this once on login. Poisoner uses CLEU for buff tracking afterwards for more CPU efficiency (filtering)
end

function Poisoner:PLAYER_REGEN_ENABLED()
	Poisoner.InCombatLockdown = false
	if Poisoner.NeedUpdate then
		Poisoner:CreateButtons()
	end
end
function Poisoner:PLAYER_REGEN_DISABLED()
	Poisoner.InCombatLockdown = true
end

function Poisoner:SPELLS_CHANGED()
	Poisoner:CreateButtons()
end

function Poisoner:COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, auraType, amount)
	
	debug("Poisoner CLEU")
	if destGUID ~= playerGUID then debug("1") return end
	if not spells[spellId] then debug("2") return end
	if not CLEU_events[event] then debug("3") return end
	
	if event == "SPELL_AURA_APPLIED" then
		currentbuffs[spellId] = Poisoner:GetBuffDuration(spellId, spellName)
	elseif event == "SPELL_AURA_REMOVED" then
		if currentbuffs[spellId] then
			currentbuffs[spellId] = nil
		end
	end
	
end


function Poisoner:GetSpellTexture(spellId)
	local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellId)
	return icon
end

function Poisoner:GetAllSpellTextures()
	for k,v in pairs(spells) do
		print(Poisoner:GetSpellTexture(k))
	end
end

function Poisoner:CreateFrames()
	Poisoner:CreateMainButton()
	Poisoner:CreateMenu()
	Poisoner:CreateButtons()
end

PoisonerTempTable = {
	frames = {
		FreeButton = {},
		AuraButton = {},
	},
}


-- local frame functions
local function modkeys()
	local shift, ctrl, alt = IsShiftKeyDown(), IsControlKeyDown(), IsAltKeyDown()
	local modkey = ""
			
	if shift then modkey = modkey.."shift" end
	if ctrl then modkey = modkey.."ctrl" end
	if alt then modkey = modkey.."alt" end
	--print("ReqKey", reqmod,"- ModKey:", modkey)
	
	return modkey
end

local function savepos(frame)
	local p, f, rp, x, y = frame:GetPoint()
	if f and f:GetName() == "UIParent" then f = "UIParent" end
	local t = {p, f or "UIParent", rp, x, y}
	PoisonerTempTable.frames[frame].position = t
end

local function tooltipShow(owner, direction, typ, value)
	--print(owner:GetName(), direction, typ, value, type(value))
	if typ == "text" then
		GameTooltip:SetOwner(owner, direction or "BOTTOMLEFT")
		GameTooltip:SetText(value)
	elseif typ == "spellid" or typ == "itemid" then
		GameTooltip:SetOwner(owner, direction or "BOTTOMLEFT")
		GameTooltip:SetHyperlink(GetSpellLink(value))
	else
		print("ELSE return")
		return
	end
	GameTooltip:Show()
end
local function tooltipHide()
	GameTooltip:ClearLines()
	GameTooltip:Hide()
end


-- Classic Menu

function Poisoner:CreateMainButton()
	local pfb = _G["Poisoner_FreeButton"]
	if pfb == nil then
		pfb = CreateFrame("BUTTON", "Poisoner_FreeButton", UIParent, "SecureHandlerClickTemplate, SecureHandlerBaseTemplate, SecureHandlerEnterLeaveTemplate, SecureHandlerMouseWheelTemplate");
		
		pfb:RegisterForDrag("LeftButton");
		pfb:SetMovable();
		local modkey = "shift"
		pfb:SetScript("OnDragStart", function(self)
			if modkeys() == modkey then
				self:StartMoving()
				self.IsMoving = true
			end
		end);
		pfb:SetScript("OnDragStop", function(self)
			if self.IsMoving == true then
				self:StopMovingOrSizing()
				self.IsMoving = false
			end
		end);
		
		pfb:RegisterForClicks("AnyUp");
	--	pfb:SetClampedToScreen(true);
		pfb:SetSize(32,32)
		pfb:ClearAllPoints();
		pfb:SetPoint("CENTER",UIParent,"CENTER");
		
		local texture = textureprefix.."ability_creature_poison_06"..texturesuffix
		
		pfb.Texture = pfb:CreateTexture(pfb:GetName().."Icon", "ARTWORK");
		pfb.Texture:SetTexture(texture);
		pfb.Texture:SetAllPoints(pfb);
		
		pfb:SetNormalTexture(texture);
		pfb:SetHighlightTexture(texture);
		
		if true then
			SetPortraitToTexture(pfb:GetNormalTexture(), texture)
			SetPortraitToTexture(pfb:GetHighlightTexture(), texture)
			SetPortraitToTexture(_G[pfb:GetName().."Icon"], texture)
		end
		
		pfb:SetAttribute("modkey", "shift")
		
		local function onenter()
			tooltipShow(Poisoner_FreeButton, "BOTTOMRIGHT", "text", "CLICK  |cffffffffto toggle the menu|r \nSHIFT and DRAG  |cffffffffto move the button around (even in combat)|r \nSHIFT and MOUSEWHEEL  |cffffffffto scale the button from 0.1 to 2.0 (even in combat)|r ")
		end
		
		pfb.OnEnter = onenter
		pfb.OnLeave = tooltipHide
		
		pfb:SetAttribute("_onclick", [=[
			local left, right, middle, menu = button == "LeftButton", button == "RightButton", button == "MiddleButton", self:GetFrameRef("menu");
			
			if right then
				--[[
				if options:IsVisible() then
						options:Hide();
				elseif options:IsShown() then
					options:Hide();
				else
					options:Show();
				end
				]]
			elseif left then
				if menu:IsVisible() then
					menu:Hide();
				elseif menu:IsShown() then
					menu:Hide();
				else
					menu:Show();
				end
			elseif middle then
				print("QuickButton updated.")
			end
			
			
		]=]);
		
		pfb:SetAttribute("_onmousewheel", [=[
			local up, down, menu = delta == 1, delta == -1, self:GetFrameRef("menu");
			local reqmod, modkey = self:GetAttribute("modkey"), ""
			local shift, ctrl, alt = IsShiftKeyDown(), IsControlKeyDown(), IsAltKeyDown()
			
			if shift then modkey = modkey.."shift" end
			if ctrl then modkey = modkey.."ctrl" end
			if alt then modkey = modkey.."alt" end
			--print("ReqKey", reqmod,"- ModKey:", modkey)
			
			if modkey ~= reqmod then return end
			
			local sScale = self:GetScale()
			local mScale = menu:GetScale()
			local ofsScale = 0.1
			
			if up and (sScale <= 2 or mScale <= 2) then
				self:SetScale(sScale + ofsScale)
				menu:SetScale(mScale + ofsScale)
			elseif down and (sScale >= 0.1 or mScale >= 0.1) then
				self:SetScale(sScale - ofsScale)
				menu:SetScale(mScale - ofsScale)
			end
			
		]=])
		
		
		pfb:SetAttribute("_onenter", [=[
			--[[
			local menu, psf = self:GetFrameRef("frame1"), self:GetFrameRef("PoisonSlotFrame");
			
			local som = psf:GetAttribute("showmouseover")
			
			if som then
				if menu:IsVisible() then
					menu:Hide();
				elseif menu:IsShown() then
					menu:Hide();
				else
					menu:Show();
				end
			end
			]]
			control:CallMethod("OnEnter", self)
		]=])
		pfb:SetAttribute("_onleave", [=[
			--[[
			local menu, psf = self:GetFrameRef("frame1"), self:GetFrameRef("PoisonSlotFrame");
			local som = psf:GetAttribute("showmouseover")
			
			if som then
				menu:Hide();
			end
			]]
			control:CallMethod("OnLeave")
		]=])
		
		if true then return end
		pfb:SetAttribute("_onclick", [=[
			local left, right, middle, menu, options, qb, psf = button == "LeftButton", button == "RightButton", button == "MiddleButton", self:GetFrameRef("frame1"), self:GetFrameRef("frame2"), self:GetFrameRef("button1"), self:GetFrameRef("PoisonSlotFrame");
			local overwrite = psf:GetAttribute("overwrite")
			
			if right then
				--[[
				if options:IsVisible() then
						options:Hide();
				elseif options:IsShown() then
					options:Hide();
				else
					options:Show();
				end
				]]
			elseif left then
				if menu:IsVisible() then
					menu:Hide();
				elseif menu:IsShown() then
					menu:Hide();
				else
					menu:Show();
				end
			elseif middle then
				--print("QuickButton updated.")
			end
			
			local poison1, poison2, poison3, poison4, poison5, poison6, poison7, poison8, poison9, poison10, poison11, poison12
			poison1 = psf:GetAttribute("Text1")
			poison2 = psf:GetAttribute("Text2")
			poison3 = psf:GetAttribute("Text3")
			poison4 = psf:GetAttribute("Text4")
			poison5 = psf:GetAttribute("Text5")
			poison6 = psf:GetAttribute("Text6")
			poison7 = psf:GetAttribute("Text7")
			poison8 = psf:GetAttribute("Text8")
			poison9 = psf:GetAttribute("Text9")
			poison10 = psf:GetAttribute("Text10")
			poison11 = psf:GetAttribute("Text11")
			poison12 = psf:GetAttribute("Text12")
			
			--print("Poisons:\n"..poison1.."\n"..poison2.."\n"..poison3.."\n"..poison4.."\n"..poison5.."\n"..poison6.."\n"..poison7.."\n"..poison8.."\n"..poison9.."\n"..poison10.."\n"..poison11.."\n"..poison12)
			
			--Poisoner_QuickButton_Text = "/use [nomodifier,button:3] "..poison9.."\n/use [modifier:shift,button:3] "..poison10.."\n/use [modifier:ctrl,button:3] "..poison11.."\n/use [modifier:alt,button:3] "..poison12.."\n/use [button:3] 18\n/use [nomodifier,button:2] "..poison5.."\n/use [modifier:shift,button:2] "..poison6.."\n/use [modifier:ctrl,button:2] "..poison7.."\n/use [modifier:alt,button:2] "..poison8.."\n/use [button:2] 17\n/use [nomodifier,button:1] "..poison1.."\n/use [modifier:shift,button:1] "..poison2.."\n/use [modifier:ctrl,button:1] "..poison3.."\n/use [modifier:alt,button:1] "..poison4.."\n/use [button:1] 16\n/click StaticPopup1Button1\n/run PoisonerOptions_CheckPoisons()"
			--print(Poisoner_QuickButton_Text)	
			--qb:SetAttribute("type", "macro");
			--qb:SetAttribute("macrotext", "TEST");
			if overwrite then
				qb:SetAttribute("macrotext", "/use [nomodifier,button:3] "..poison9.."\n/use [modifier:shift,button:3] "..poison10.."\n/use [modifier:ctrl,button:3] "..poison11.."\n/use [modifier:alt,button:3] "..poison12.."\n/use [button:3] 18\n/use [nomodifier,button:2] "..poison5.."\n/use [modifier:shift,button:2] "..poison6.."\n/use [modifier:ctrl,button:2] "..poison7.."\n/use [modifier:alt,button:2] "..poison8.."\n/use [button:2] 17\n/use [nomodifier,button:1] "..poison1.."\n/use [modifier:shift,button:1] "..poison2.."\n/use [modifier:ctrl,button:1] "..poison3.."\n/use [modifier:alt,button:1] "..poison4.."\n/use [button:1] 16\n/click StaticPopup1Button1\n/run PoisonerOptions_CheckPoisons()");
			end
			
		]=]);
	end
end

function Poisoner:CreateMenu()
	local pfb = _G["Poisoner_FreeButton"] or Poisoner:CreateMainButton()
	local pmenu = _G["Poisoner_Menu"]
	if pmenu == nil then
		pmenu = CreateFrame("Frame", "Poisoner_Menu", UIParent, "SecureHandlerEnterLeaveTemplate");
		pmenu:SetSize(64*4,64)
		pmenu:ClearAllPoints();
		pmenu:SetPoint("RIGHT",Poisoner_FreeButton,"LEFT");
		--[[
		local texture = textureprefix.."ability_creature_poison_06"..texturesuffix
		pmenu.Texture = pmenu:CreateTexture(pmenu:GetName().."Icon", "ARTWORK");
		pmenu.Texture:SetTexture(texture);
		pmenu.Texture:SetAllPoints();
		]]		
		_G["Poisoner_FreeButton"]:SetFrameRef("menu", Poisoner_Menu);
	end
	return pmenu
end

function Poisoner:CreateButtons()
	if Poisoner.InCombatLockdown or InCombatLockdown() then Poisoner.NeedUpdate = true return end
	
	local pmenu = Poisoner:CreateMenu()
	local pmenuwidth = 0
	local prevbutton
	for cat,spells in pairs(poisons) do
		prevbutton = nil
		for spellID,v in pairs(spells) do
			local buttonName = "Poisoner_MenuButton__"..spellID;
			local button = _G[buttonName];
			if IsSpellKnown(spellID) then
				
				if button == nil then
					
					button = CreateFrame("Button", buttonName, pmenu, "SecureActionButtonTemplate, SecureHandlerEnterLeaveTemplate, SecureHandlerAttributeTemplate,SecureHandlerDragTemplate,SecureHandlerMouseUpDownTemplate")
					button:SetHeight(32)
					button:SetWidth(32)
					--button:ClearAllPoints();
					--button:SetPoint("CENTER",UIParent,"CENTER");
					
					button.Name = GetSpellInfo(spellID)
					button.spellID = spellID
					
					local name, rank, icon = GetSpellInfo(spellID)
					
					button:SetAttribute("name", name);
					button:SetAttribute("type", "macro");
					button:SetAttribute("macrotext", "/cast "..name)
					
					local texture = gsub(icon, "Interface\\Icons\\", "Interface\\AddOns\\Poisoner\\images\\")
					
					button:SetNormalTexture(texture);
					button:SetHighlightTexture(texture);
					
					button.Texture = button:CreateTexture(button:GetName().."Icon", "ARTWORK");
					button.Texture:SetTexture(texture);
					button.Texture:SetAllPoints(button);
					
					if true then
						SetPortraitToTexture(button:GetNormalTexture(), texture)
						SetPortraitToTexture(button:GetHighlightTexture(), texture)
						SetPortraitToTexture(_G[button:GetName().."Icon"], texture)
					end
					
					local bfs = button:CreateFontString(buttonName.."Text","OVERLAY","GameFontNormal");
					bfs:SetPoint("BOTTOMRIGHT");
					bfs:SetTextColor(1,1,1,1);
					bfs:SetShadowColor(0,0,0,1);
					bfs:SetShadowOffset(-2,2);
					button:SetFontString(bfs);
					
					button:RegisterForClicks("AnyDown");
					
					button:SetScript("OnEnter", function(self) tooltipShow(self, nil, "spellid", self.spellID) end)
					button:SetScript("OnLeave", function() tooltipHide() end)
					
				end
				
				-- >>> Visuals
					local nummodels = 3
					if true then
						for i=1,nummodels do
							local Model = _G[buttonName.."Model"..i]
							if not Model then
								Model = CreateFrame("Model", buttonName.."Model"..i, button);
							end
							Model:SetAllPoints( nil ); -- Fullscreen
							Model:Show();
							Model:SetLight( 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 );
							Model:SetModel("spells\\lightning_fel_precast_low_hand.mdx")
							Model:SetModelScale(0.02)
							Model:SetFacing(0)
							Model:SetFrameStrata(button:GetFrameStrata())
						end
						
						local UIScale = UIParent:GetEffectiveScale()
						local Hypotenuse = ( GetScreenWidth() ^ 2 + GetScreenHeight() ^ 2 ) ^ 0.5 * UIScale
						local roundtime, starttime = 2000, GetTime()*1000
						local endtime = starttime + roundtime
						
						button:SetScript("OnUpdate", function()
							local X,Y
							X = ( (button:GetRight() or 0) + (button:GetLeft() or 0) ) /2
							Y = ( (button:GetTop() or 0) + (button:GetBottom() or 0) ) /2
							local currenttime = GetTime()*1000
							for i=1,nummodels do
								local Model = _G[buttonName.."Model"..i]
								if endtime >= currenttime then
									Model:Show()
									local ofsa = (360 / nummodels) * i
									local per = (endtime - currenttime) / ((endtime - starttime) / 100)
									local angle = 360/100*per + ofsa
									local ofsx, ofsy = GetCircularPoint(angle, 16)
									Model:SetPosition( ( X + ofsx) / Hypotenuse, ( Y + ofsy) / Hypotenuse, 0 );
								else
									starttime = currenttime
									endtime = currenttime + roundtime
								end
								if currentbuffs[button.spellID] then
									Model:SetModelScale(0.02)
								else
									Model:SetModelScale(0.0)
								end
							end
						end)
						
					end
				-- <<< Visuals
				
				button:ClearAllPoints();
				if prevbutton then
					button:SetPoint("RIGHT", prevbutton, "LEFT");
				else
					if cat == "Lethal" then
						button:SetPoint("TOPRIGHT", pmenu, "TOPRIGHT");
					else
						button:SetPoint("BOTTOMRIGHT", pmenu, "BOTTOMRIGHT");
					end
				end
				button:Show()
				prevbutton = button
				
				pmenuwidth = pmenuwidth + button:GetWidth()
				
			else
				if button then
					button:Hide()
					button:SetScript("OnUpdate", nil)
				end
			end
		end		
	end
	
	pmenu:SetWidth(pmenuwidth)
	
end



--[[
	TIMER
]]
-- lookup the duration of the spells

function Poisoner:CheckBuffs()
	currentbuffs = {}
	local currentTime = GetTime()*1000
	for k,v in pairs(spells) do
		local spellName = GetSpellInfo(k)
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3
		if spellName then
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", spellName)
		end
		if name then
			currentbuffs[k] = expirationTime*1000
		end
	end
	
	return currentbuffs
end

function Poisoner:GetAllBuffDurations()
	if currentbuffs == {} then
		currentbuffs = Poisoner:CheckBuffs()
	end
	
	return currentbuffs
end

function Poisoner:GetBuffDuration(reqspellId, spellName)
	if not tonumber(reqspellId) then return nil end
	local currentTime = GetTime()*1000
	local bufftime	
	local spellName = spellName or GetSpellInfo(reqspellId)
	local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3
	if spellName then
		name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3 = UnitBuff("player", spellName)
	end
	if name and spellId == reqspellId then
		bufftime = expirationTime*1000
	end
		
	return bufftime
end


function Poisoner:Test()
	print("Poisoner TEST")
	print("Poisoner playerGUID = ", playerGUID)
end

